@@ -1,7 +1,6 @@
-import sys
 from ctypes import Structure, c_int, c_char_p, c_double, c_void_p, CFUNCTYPE, \
     POINTER
-from .dll import _bind
+from .dll import _bind, nullfunc
 from .endian import SDL_BYTEORDER, SDL_LIL_ENDIAN
 from .stdinc import Uint8, Uint16, Uint32
 from .rwops import SDL_RWops, SDL_RWFromFile
@@ -14,7 +13,7 @@ __all__ = ["SDL_AudioFormat", "SDL_AUDIO_MASK_BITSIZE",
            "AUDIO_S8", "AUDIO_U16LSB", "AUDIO_S16LSB", "AUDIO_U16MSB",
            "AUDIO_S16MSB", "AUDIO_U16", "AUDIO_S16", "AUDIO_S32LSB",
            "AUDIO_S32MSB", "AUDIO_S32", "AUDIO_F32LSB", "AUDIO_S32MSB",
-           "AUDIO_F32", "AUDIO_U16SYS", "AUDIO_S16SYS", "AUDIO_S32SYS",
+           "AUDIO_F32", "AUDIO_U16SYS", "AUDIO_S16SYS", "AUDIO_S32SYS", "AUDIO_FORMATS",
            "AUDIO_F32SYS", "SDL_AUDIO_ALLOW_FREQUENCY_CHANGE",
            "SDL_AUDIO_ALLOW_FORMAT_CHANGE", "SDL_AUDIO_ALLOW_CHANNELS_CHANGE",
            "SDL_AUDIO_ALLOW_ANY_CHANGE", "SDL_AudioCallback", "SDL_AudioSpec",
@@ -29,7 +28,8 @@ __all__ = ["SDL_AudioFormat", "SDL_AUDIO_MASK_BITSIZE",
            "SDL_ConvertAudio", "SDL_MIX_MAXVOLUME", "SDL_MixAudio",
            "SDL_MixAudioFormat", "SDL_LockAudio", "SDL_LockAudioDevice",
            "SDL_UnlockAudio", "SDL_UnlockAudioDevice", "SDL_CloseAudio",
-           "SDL_CloseAudioDevice"
+           "SDL_CloseAudioDevice", "SDL_QueueAudio", "SDL_GetQueuedAudioSize",
+           "SDL_ClearQueuedAudio"
          ]
 
 SDL_AudioFormat = Uint16
@@ -61,6 +61,15 @@ AUDIO_F32LSB = 0x8120
 AUDIO_F32MSB = 0x9120
 AUDIO_F32 = AUDIO_F32LSB
 
+
+# All of the audio formats should be in this set which is provided as a
+# convenience to the end user for purposes of iteration and validation.
+# (is the provided audio format in the supported set?)
+AUDIO_FORMATS = set([AUDIO_U8, AUDIO_S8, AUDIO_U16LSB, AUDIO_S16LSB,
+                     AUDIO_U16MSB, AUDIO_S16MSB, AUDIO_U16, AUDIO_S16,
+                     AUDIO_S32LSB, AUDIO_S32MSB, AUDIO_S32, AUDIO_F32LSB,
+                     AUDIO_F32MSB, AUDIO_F32])
+
 if SDL_BYTEORDER == SDL_LIL_ENDIAN:
     AUDIO_U16SYS = AUDIO_U16LSB
     AUDIO_S16SYS = AUDIO_S16LSB
@@ -106,14 +115,6 @@ class SDL_AudioCVT(Structure):
     pass
 
 SDL_AudioFilter = CFUNCTYPE(POINTER(SDL_AudioCVT), SDL_AudioFormat)
-# HACK: hack for an IronPython 2.7.2.1+ issue:
-#    ptrarray = (CFUNCTYPE() * int)
-# is not supported properly
-if sys.platform == "cli":
-    _X_SDL_AudioFilter = POINTER(SDL_AudioFilter)
-else:
-    _X_SDL_AudioFilter = SDL_AudioFilter
-
 SDL_AudioCVT._fields_ = [("needed", c_int),
                          ("src_format", SDL_AudioFormat),
                          ("dst_format", SDL_AudioFormat),
@@ -123,7 +124,7 @@ SDL_AudioCVT._fields_ = [("needed", c_int),
                          ("len_cvt", c_int),
                          ("len_mult", c_int),
                          ("len_ratio", c_double),
-                         ("filters", (_X_SDL_AudioFilter * 10)),
+                         ("filters", (SDL_AudioFilter * 10)),
                          ("filter_index", c_int)
                          ]
 
@@ -146,7 +147,7 @@ SDL_GetAudioDeviceStatus = _bind("SDL_GetAudioDeviceStatus", [SDL_AudioDeviceID]
 SDL_PauseAudio = _bind("SDL_PauseAudio", [c_int])
 SDL_PauseAudioDevice = _bind("SDL_PauseAudioDevice", [SDL_AudioDeviceID, c_int])
 SDL_LoadWAV_RW = _bind("SDL_LoadWAV_RW", [POINTER(SDL_RWops), c_int, POINTER(SDL_AudioSpec), POINTER(POINTER(Uint8)), POINTER(Uint32)], POINTER(SDL_AudioSpec))
-SDL_LoadWAV = lambda f, s, ab, al: SDL_LoadWAV_RW(SDL_RWFromFile(f, "rb"), 1, s, ab , al)
+SDL_LoadWAV = lambda f, s, ab, al: SDL_LoadWAV_RW(SDL_RWFromFile(f, b"rb"), 1, s, ab , al)
 SDL_FreeWAV = _bind("SDL_FreeWAV", [POINTER(Uint8)])
 SDL_BuildAudioCVT = _bind("SDL_BuildAudioCVT", [POINTER(SDL_AudioCVT), SDL_AudioFormat, Uint8, c_int, SDL_AudioFormat, Uint8, c_int], c_int)
 SDL_ConvertAudio = _bind("SDL_ConvertAudio", [POINTER(SDL_AudioCVT)], c_int)
@@ -159,4 +160,7 @@ SDL_UnlockAudio = _bind("SDL_UnlockAudio")
 SDL_UnlockAudioDevice = _bind("SDL_UnlockAudioDevice", [SDL_AudioDeviceID])
 SDL_CloseAudio = _bind("SDL_CloseAudio")
 SDL_CloseAudioDevice = _bind("SDL_CloseAudioDevice", [SDL_AudioDeviceID])
+SDL_QueueAudio = _bind("SDL_QueueAudio", [SDL_AudioDeviceID, c_void_p, Uint32], c_int, nullfunc)
+SDL_GetQueuedAudioSize = _bind("SDL_GetQueuedAudioSize", [SDL_AudioDeviceID], Uint32, nullfunc)
+SDL_ClearQueuedAudio = _bind("SDL_ClearQueuedAudio", [SDL_AudioDeviceID], optfunc=nullfunc)
 
