@@ -34,10 +34,8 @@ class SDLHintsTest(unittest.TestCase):
         self.assertEqual(hints.SDL_GetHint(b"TEST"), b"32")
         self.assertEqual(hints.SDL_SetHint(b"TEST", b"abcdef"), 1)
         self.assertEqual(hints.SDL_GetHint(b"TEST"), b"abcdef")
-        if sys.platform != "cli":
-            # TODO: Check on next IronPython version (>2.7.4)
-            self.assertEqual(hints.SDL_SetHint(b"", b""), 1)
-            self.assertEqual(hints.SDL_GetHint(b""), b"")
+        self.assertEqual(hints.SDL_SetHint(b"", b""), 1)
+        self.assertEqual(hints.SDL_GetHint(b""), b"")
 
     def test_SDL_SetHintWithPriority(self):
         self.assertEqual(hints.SDL_SetHintWithPriority
@@ -46,11 +44,9 @@ class SDLHintsTest(unittest.TestCase):
         self.assertEqual(hints.SDL_SetHintWithPriority
                          (b"TEST", b"abcdef", hints.SDL_HINT_NORMAL), 1)
         self.assertEqual(hints.SDL_GetHint(b"TEST"), b"abcdef")
-        if sys.platform != "cli":
-            # TODO: Check on next IronPython version (>2.7.4)
-            self.assertEqual(hints.SDL_SetHintWithPriority
-                             (b"", b"", hints.SDL_HINT_OVERRIDE), 1)
-            self.assertEqual(hints.SDL_GetHint(b""), b"")
+        self.assertEqual(hints.SDL_SetHintWithPriority
+                         (b"", b"", hints.SDL_HINT_OVERRIDE), 1)
+        self.assertEqual(hints.SDL_GetHint(b""), b"")
 
 
         # self.assertRaises(ValueError, hints.SDL_SetHintWithPriority,
