@@ -14,6 +14,11 @@ _ISPYPY = hasattr(sys, "pypy_version_info")
 
 RESOURCES = Resources(__file__, "resources")
 
+if _ISPYPY:
+    import gc
+    dogc = gc.collect
+else:
+    dogc = lambda: None
 
 class MSprite(sdl2ext.Sprite):
     def __init__(self, w=0, h=0):
@@ -50,6 +55,39 @@ class SDL2ExtSpriteTest(unittest.TestCase):
                     self.assertTrue(view[y][x] in c2,
                                     msg % (x, y, view[y][x], c2))
 
+    def check_areas(self, view, w, h, rects, c1, c2):
+        def _inarea(x, y, rs):
+            for r in rs:
+                if (x >= r[0] and x < (r[0] + r[2]) and
+                    y >= r[1] and y < (r[1] + r[3])):
+                    return True
+            return False
+        msg = "color mismatch at %d,%d: %d not in %s"
+        for y in range(w):
+            for x in range(h):
+                if _inarea(x, y, rects):
+                    self.assertEqual(view[y][x], c1,
+                                     msg % (x, y, view[y][x], c1))
+                else:
+                    self.assertTrue(view[y][x] in c2,
+                                    msg % (x, y, view[y][x], c2))
+
+    def check_lines(self, view, w, h, points, c1, c2):
+        def _online(x, y, pts):
+            for p1, p2 in pts:
+                if sdl2ext.point_on_line(p1, p2, (x, y)):
+                    return True
+            return False
+        msg = "color mismatch at %d,%d: %d not in %s"
+        for y in range(w):
+            for x in range(h):
+                if _online(x, y, points):
+                    self.assertEqual(view[y][x], c1,
+                                     msg % (x, y, view[y][x], c1))
+                else:
+                    self.assertTrue(view[y][x] in c2,
+                                    msg % (x, y, view[y][x], c2))
+
     def test_SpriteFactory(self):
         factory = sdl2ext.SpriteFactory(sdl2ext.SOFTWARE)
         self.assertIsInstance(factory, sdl2ext.SpriteFactory)
@@ -60,7 +98,7 @@ class SDL2ExtSpriteTest(unittest.TestCase):
         self.assertEqual(factory.default_args, {"bananas": "tasty"})
 
         window = sdl2ext.Window("Test", size=(1, 1))
-        renderer = sdl2ext.RenderContext(window)
+        renderer = sdl2ext.Renderer(window)
 
         factory = sdl2ext.SpriteFactory(sdl2ext.TEXTURE, renderer=renderer)
         self.assertIsInstance(factory, sdl2ext.SpriteFactory)
@@ -73,10 +111,11 @@ class SDL2ExtSpriteTest(unittest.TestCase):
         self.assertRaises(ValueError, sdl2ext.SpriteFactory, -456)
         self.assertRaises(ValueError, sdl2ext.SpriteFactory, 123)
         self.assertRaises(ValueError, sdl2ext.SpriteFactory, sdl2ext.TEXTURE)
+        dogc()
 
     def test_SpriteFactory_create_sprite(self):
         window = sdl2ext.Window("Test", size=(1, 1))
-        renderer = sdl2ext.RenderContext(window)
+        renderer = sdl2ext.Renderer(window)
         tfactory = sdl2ext.SpriteFactory(sdl2ext.TEXTURE, renderer=renderer)
         sfactory = sdl2ext.SpriteFactory(sdl2ext.SOFTWARE)
 
@@ -92,6 +131,7 @@ class SDL2ExtSpriteTest(unittest.TestCase):
                     continue
                 sprite = tfactory.create_sprite(size=(w, h))
                 self.assertIsInstance(sprite, sdl2ext.TextureSprite)
+        dogc()
 
     def test_SpriteFactory_create_software_sprite(self):
         factory = sdl2ext.SpriteFactory(sdl2ext.SOFTWARE)
@@ -114,10 +154,11 @@ class SDL2ExtSpriteTest(unittest.TestCase):
         self.assertRaises((ArgumentError, TypeError),
                           factory.create_software_sprite, size=(10, 10),
                           masks=("Test", 1, 2, 3))
+        dogc()
 
     def test_SpriteFactory_create_texture_sprite(self):
         window = sdl2ext.Window("Test", size=(1, 1))
-        renderer = sdl2ext.RenderContext(window)
+        renderer = sdl2ext.Renderer(window)
         factory = sdl2ext.SpriteFactory(sdl2ext.TEXTURE, renderer=renderer)
         for w in range(1, 100):
             for h in range(1, 100):
@@ -130,10 +171,11 @@ class SDL2ExtSpriteTest(unittest.TestCase):
             sprite = factory.create_texture_sprite(renderer, size=(64, 64),
                                                    access=flag)
             self.assertIsInstance(sprite, sdl2ext.TextureSprite)
+        dogc()
 
     def test_SpriteFactory_from_image(self):
         window = sdl2ext.Window("Test", size=(1, 1))
-        renderer = sdl2ext.RenderContext(window)
+        renderer = sdl2ext.Renderer(window)
         tfactory = sdl2ext.SpriteFactory(sdl2ext.TEXTURE, renderer=renderer)
         sfactory = sdl2ext.SpriteFactory(sdl2ext.SOFTWARE)
 
@@ -145,23 +187,24 @@ class SDL2ExtSpriteTest(unittest.TestCase):
             self.assertIsInstance(ssprite, sdl2ext.SoftwareSprite)
 
         for factory in (tfactory, sfactory):
-            self.assertRaises((AttributeError, TypeError, sdl2ext.SDLError),
+            self.assertRaises((ArgumentError, ValueError),
                               factory.from_image, None)
             #self.assertRaises((IOError, SDLError),
             #                  factory.from_image, "banana")
-            self.assertRaises((AttributeError, IOError, sdl2ext.SDLError),
-                              factory.from_image, 12345)
+            if not _ISPYPY:
+                self.assertRaises(ArgumentError, factory.from_image, 12345)
+        dogc()
 
     @unittest.skip("not implemented")
     def test_SpriteFactory_from_object(self):
         window = sdl2ext.Window("Test", size=(1, 1))
-        renderer = sdl2ext.RenderContext(window)
+        renderer = sdl2ext.Renderer(window)
         tfactory = sdl2ext.SpriteFactory(sdl2ext.TEXTURE, renderer=renderer)
         sfactory = sdl2ext.SpriteFactory(sdl2ext.SOFTWARE)
 
     def test_SpriteFactory_from_surface(self):
         window = sdl2ext.Window("Test", size=(1, 1))
-        renderer = sdl2ext.RenderContext(window)
+        renderer = sdl2ext.Renderer(window)
         tfactory = sdl2ext.SpriteFactory(sdl2ext.TEXTURE, renderer=renderer)
         sfactory = sdl2ext.SpriteFactory(sdl2ext.SOFTWARE)
 
@@ -180,6 +223,7 @@ class SDL2ExtSpriteTest(unittest.TestCase):
             # TODO: crashes pypy 2.0
             #self.assertRaises((AttributeError, ArgumentError, TypeError),
             #                  factory.from_surface, 1234)
+        dogc()
 
     def test_SpriteFactory_from_text(self):
         sfactory = sdl2ext.SpriteFactory(sdl2ext.SOFTWARE)
@@ -197,24 +241,25 @@ class SDL2ExtSpriteTest(unittest.TestCase):
 
         # Get text from a texture sprite factory
         window = sdl2ext.Window("Test", size=(1, 1))
-        renderer = sdl2ext.RenderContext(window)
+        renderer = sdl2ext.Renderer(window)
         tfactory = sdl2ext.SpriteFactory(sdl2ext.TEXTURE,
                                          renderer=renderer,
                                          fontmanager=fm)
         sprite = tfactory.from_text("Test", alias="tuffy")
         self.assertIsInstance(sprite, sdl2ext.TextureSprite)
+        dogc()
 
-    def test_SpriteRenderer(self):
-        renderer = sdl2ext.SpriteRenderer()
-        self.assertIsInstance(renderer, sdl2ext.SpriteRenderer)
+    def test_SpriteRenderSystem(self):
+        renderer = sdl2ext.SpriteRenderSystem()
+        self.assertIsInstance(renderer, sdl2ext.SpriteRenderSystem)
         self.assertIsNotNone(renderer.sortfunc)
         self.assertTrue(sdl2ext.Sprite in renderer.componenttypes)
 
-    def test_SpriteRenderer_sortfunc(self):
+    def test_SpriteRenderSystem_sortfunc(self):
         def func(p):
             pass
 
-        renderer = sdl2ext.SpriteRenderer()
+        renderer = sdl2ext.SpriteRenderSystem()
         self.assertIsNotNone(renderer.sortfunc)
         renderer.sortfunc = func
         self.assertEqual(renderer.sortfunc, func)
@@ -226,36 +271,39 @@ class SDL2ExtSpriteTest(unittest.TestCase):
         self.assertRaises(TypeError, setf, renderer, 1234)
 
     @unittest.skip("not implemented")
-    def test_SpriteRenderer_render(self):
+    def test_SpriteRenderSystem_render(self):
         pass
 
     @unittest.skip("not implemented")
-    def test_SpriteRenderer_process(self):
+    def test_SpriteRenderSystem_process(self):
         pass
 
-    def test_SoftwareSpriteRenderer(self):
-        self.assertRaises(TypeError, sdl2ext.SoftwareSpriteRenderer)
-        self.assertRaises(TypeError, sdl2ext.SoftwareSpriteRenderer, None)
-        self.assertRaises(TypeError, sdl2ext.SoftwareSpriteRenderer, "Test")
-        self.assertRaises(TypeError, sdl2ext.SoftwareSpriteRenderer, 12345)
+    def test_SoftwareSpriteRenderSystem(self):
+        self.assertRaises(TypeError, sdl2ext.SoftwareSpriteRenderSystem)
+        self.assertRaises(TypeError, sdl2ext.SoftwareSpriteRenderSystem, None)
+        self.assertRaises(TypeError, sdl2ext.SoftwareSpriteRenderSystem,
+                          "Test")
+        self.assertRaises(TypeError, sdl2ext.SoftwareSpriteRenderSystem,
+                          12345)
 
         window = sdl2ext.Window("Test", size=(1, 1))
-        renderer = sdl2ext.SoftwareSpriteRenderer(window)
-        self.assertIsInstance(renderer, sdl2ext.SpriteRenderer)
+        renderer = sdl2ext.SoftwareSpriteRenderSystem(window)
+        self.assertIsInstance(renderer, sdl2ext.SpriteRenderSystem)
         self.assertEqual(renderer.window, window.window)
         self.assertIsInstance(renderer.surface, SDL_Surface)
 
-        renderer = sdl2ext.SoftwareSpriteRenderer(window.window)
-        self.assertIsInstance(renderer, sdl2ext.SpriteRenderer)
+        renderer = sdl2ext.SoftwareSpriteRenderSystem(window.window)
+        self.assertIsInstance(renderer, sdl2ext.SpriteRenderSystem)
         self.assertEqual(renderer.window, window.window)
         self.assertIsInstance(renderer.surface, SDL_Surface)
 
         self.assertIsNotNone(renderer.sortfunc)
         self.assertFalse(sdl2ext.Sprite in renderer.componenttypes)
         self.assertTrue(sdl2ext.SoftwareSprite in renderer.componenttypes)
+        dogc()
 
     @unittest.skipIf(_ISPYPY, "PyPy's ctypes can't do byref(value, offset)")
-    def test_SoftwareSpriteRenderer_render(self):
+    def test_SoftwareSpriteRenderSystem_render(self):
         sf1 = SDL_CreateRGBSurface(0, 12, 7, 32, 0, 0, 0, 0)
         sp1 = sdl2ext.SoftwareSprite(sf1.contents, True)
         sdl2ext.fill(sp1, 0xFF0000)
@@ -266,8 +314,8 @@ class SDL2ExtSpriteTest(unittest.TestCase):
         sprites = [sp1, sp2]
 
         window = sdl2ext.Window("Test", size=(20, 20))
-        renderer = sdl2ext.SoftwareSpriteRenderer(window)
-        self.assertIsInstance(renderer, sdl2ext.SpriteRenderer)
+        renderer = sdl2ext.SoftwareSpriteRenderSystem(window)
+        self.assertIsInstance(renderer, sdl2ext.SpriteRenderSystem)
 
         self.assertRaises(AttributeError, renderer.render, None, None, None)
         self.assertRaises(AttributeError, renderer.render, [None, None],
@@ -295,7 +343,7 @@ class SDL2ExtSpriteTest(unittest.TestCase):
         del view
 
     @unittest.skipIf(_ISPYPY, "PyPy's ctypes can't do byref(value, offset)")
-    def test_SoftwareSpriteRenderer_process(self):
+    def test_SoftwareSpriteRenderSystem_process(self):
         sf1 = SDL_CreateRGBSurface(0, 5, 10, 32, 0, 0, 0, 0)
         sp1 = sdl2ext.SoftwareSprite(sf1.contents, True)
         sp1.depth = 0
@@ -308,7 +356,7 @@ class SDL2ExtSpriteTest(unittest.TestCase):
         sprites = [sp1, sp2]
 
         window = sdl2ext.Window("Test", size=(20, 20))
-        renderer = sdl2ext.SoftwareSpriteRenderer(window)
+        renderer = sdl2ext.SoftwareSpriteRenderSystem(window)
 
         renderer.process("fakeworld", sprites)
         view = sdl2ext.PixelView(renderer.surface)
@@ -320,15 +368,15 @@ class SDL2ExtSpriteTest(unittest.TestCase):
         self.assertRaises(TypeError, renderer.process, None, None)
 
     @unittest.skip("not implemented")
-    def test_TextureSpriteRenderer(self):
+    def test_TextureSpriteRenderSystem(self):
         pass
 
     @unittest.skip("not implemented")
-    def test_TextureSpriteRenderer_render(self):
+    def test_TextureSpriteRenderSystem_render(self):
         pass
 
     @unittest.skip("not implemented")
-    def test_TextureSpriteRenderer_process(self):
+    def test_TextureSpriteRenderSystem_process(self):
         pass
 
     def test_Sprite(self):
@@ -423,6 +471,7 @@ class SDL2ExtSpriteTest(unittest.TestCase):
         self.assertIsInstance(sprite, sdl2ext.TextureSprite)
         SDL_DestroyRenderer(renderer)
         SDL_DestroyWindow(window)
+        dogc()
 
     def test_TextureSprite_position_xy(self):
         window = POINTER(SDL_Window)()
@@ -443,6 +492,7 @@ class SDL2ExtSpriteTest(unittest.TestCase):
             self.assertEqual(sprite.position, (x + 1, y + 1))
         SDL_DestroyRenderer(renderer)
         SDL_DestroyWindow(window)
+        dogc()
 
     def test_TextureSprite_size(self):
         window = POINTER(SDL_Window)()
@@ -459,6 +509,7 @@ class SDL2ExtSpriteTest(unittest.TestCase):
                 del sprite
         SDL_DestroyRenderer(renderer)
         SDL_DestroyWindow(window)
+        dogc()
 
     def test_TextureSprite_area(self):
         window = POINTER(SDL_Window)()
@@ -481,28 +532,104 @@ class SDL2ExtSpriteTest(unittest.TestCase):
         self.assertEqual(sprite.area, (-22, 99, -12, 119))
         SDL_DestroyRenderer(renderer)
         SDL_DestroyWindow(window)
+        dogc()
+
+    def test_Renderer(self):
+        sf = SDL_CreateRGBSurface(0, 10, 10, 32, 0, 0, 0, 0).contents
+
+        renderer = sdl2ext.Renderer(sf)
+        self.assertEqual(renderer.rendertarget, sf)
+        self.assertIsInstance(renderer.renderer.contents, SDL_Renderer)
+        del renderer
+
+        sprite = sdl2ext.SoftwareSprite(sf, True)
+        renderer = sdl2ext.Renderer(sprite)
+        self.assertEqual(renderer.rendertarget, sprite.surface)
+        self.assertEqual(renderer.rendertarget, sf)
+        self.assertIsInstance(renderer.renderer.contents, SDL_Renderer)
+        del renderer
+        dogc()
+
+        window = sdl2ext.Window("Test", size=(1, 1))
+        renderer = sdl2ext.Renderer(window)
+        self.assertEqual(renderer.rendertarget, window.window)
+        self.assertIsInstance(renderer.renderer.contents, SDL_Renderer)
+        del renderer
+        dogc()
+
+        sdlwindow = window.window
+        renderer = sdl2ext.Renderer(sdlwindow)
+        self.assertEqual(renderer.rendertarget, sdlwindow)
+        self.assertEqual(renderer.rendertarget, window.window)
+        self.assertIsInstance(renderer.renderer.contents, SDL_Renderer)
+        del renderer
+        del window
+
+        self.assertRaises(TypeError, sdl2ext.Renderer, None)
+        self.assertRaises(TypeError, sdl2ext.Renderer, 1234)
+        self.assertRaises(TypeError, sdl2ext.Renderer, "test")
+        dogc()
+
+    @unittest.skipIf(_ISPYPY, "PyPy's ctypes can't do byref(value, offset)")
+    def test_Renderer_color(self):
+        sf = SDL_CreateRGBSurface(0, 10, 10, 32,
+                                  0xFF000000,
+                                  0x00FF0000,
+                                  0x0000FF00,
+                                  0x000000FF)
+        renderer = sdl2ext.Renderer(sf.contents)
+        self.assertIsInstance(renderer.color, sdl2ext.Color)
+        self.assertEqual(renderer.color, sdl2ext.Color(0, 0, 0 ,0))
+        renderer.color = 0x00FF0000
+        self.assertEqual(renderer.color, sdl2ext.Color(0xFF, 0, 0, 0))
+        renderer.clear()
+        view = sdl2ext.PixelView(sf.contents)
+        self.check_areas(view, 10, 10, [[0, 0, 10, 10]], 0xFF000000, (0x0,))
+        del view
+        renderer.color = 0xAABBCCDD
+        self.assertEqual(renderer.color, sdl2ext.Color(0xBB, 0xCC, 0xDD, 0xAA))
+        renderer.clear()
+        view = sdl2ext.PixelView(sf.contents)
+        self.check_areas(view, 10, 10, [[0, 0, 10, 10]], 0xBBCCDDAA, (0x0,))
+        del view
+        del renderer
+        SDL_FreeSurface(sf)
+        dogc()
 
     @unittest.skip("not implemented")
-    def test_RenderContext(self):
-        pass
-
-    @unittest.skip("not implemented")
-    def test_RenderContext_color(self):
-        pass
-
-    @unittest.skip("not implemented")
-    def test_RenderContext_blendmode(self):
-        pass
-
-    @unittest.skip("not implemented")
-    def test_RenderContext_clear(self):
+    def test_Renderer_blendmode(self):
         pass
 
     @unittest.skipIf(_ISPYPY, "PyPy's ctypes can't do byref(value, offset)")
-    def test_RenderContext_copy(self):
+    def test_Renderer_clear(self):
+        sf = SDL_CreateRGBSurface(0, 10, 10, 32,
+                                  0xFF000000,
+                                  0x00FF0000,
+                                  0x0000FF00,
+                                  0x000000FF)
+        renderer = sdl2ext.Renderer(sf.contents)
+        self.assertIsInstance(renderer.color, sdl2ext.Color)
+        self.assertEqual(renderer.color, sdl2ext.Color(0, 0, 0 ,0))
+        renderer.color = 0x00FF0000
+        self.assertEqual(renderer.color, sdl2ext.Color(0xFF, 0, 0, 0))
+        renderer.clear()
+        view = sdl2ext.PixelView(sf.contents)
+        self.check_areas(view, 10, 10, [[0, 0, 10, 10]], 0xFF000000, (0x0,))
+        del view
+        renderer.clear(0xAABBCCDD)
+        self.assertEqual(renderer.color, sdl2ext.Color(0xFF, 0, 0, 0))
+        view = sdl2ext.PixelView(sf.contents)
+        self.check_areas(view, 10, 10, [[0, 0, 10, 10]], 0xBBCCDDAA, (0x0,))
+        del view
+        del renderer
+        SDL_FreeSurface(sf)
+        dogc()
+
+    @unittest.skipIf(_ISPYPY, "PyPy's ctypes can't do byref(value, offset)")
+    def test_Renderer_copy(self):
         surface = SDL_CreateRGBSurface(0, 128, 128, 32, 0, 0, 0, 0).contents
         sdl2ext.fill(surface, 0x0)
-        renderer = sdl2ext.RenderContext(surface)
+        renderer = sdl2ext.Renderer(surface)
         factory = sdl2ext.SpriteFactory(sdl2ext.TEXTURE, renderer=renderer)
         w, h = 32, 32
         sp = factory.from_color(0xFF0000, (w, h))
@@ -512,23 +639,53 @@ class SDL2ExtSpriteTest(unittest.TestCase):
         self.check_pixels(view, 128, 128, sp, 0xFF0000, (0x0,))
         del view
 
-    @unittest.skip("not implemented")
-    def test_RenderContext_draw_line(self):
-        pass
+    @unittest.skipIf(_ISPYPY, "PyPy's ctypes can't do byref(value, offset)")
+    def test_Renderer_draw_line(self):
+        surface = SDL_CreateRGBSurface(0, 128, 128, 32, 0, 0, 0, 0).contents
+        sdl2ext.fill(surface, 0x0)
+        renderer = sdl2ext.Renderer(surface)
+        renderer.draw_line((20, 10, 20, 86), 0x0000FF)
+        view = sdl2ext.PixelView(surface)
+        self.check_lines(view, 128, 128,
+                         [((20, 10), (20, 86))], 0x0000FF, (0x0,))
+        del view
 
     @unittest.skip("not implemented")
-    def test_RenderContext_draw_point(self):
-        pass
-
-    @unittest.skip("not implemented")
-    def test_RenderContext_draw_rect(self):
+    def test_Renderer_draw_point(self):
         pass
 
     @unittest.skipIf(_ISPYPY, "PyPy's ctypes can't do byref(value, offset)")
-    def test_RenderContext_fill(self):
+    def test_Renderer_draw_rect(self):
         surface = SDL_CreateRGBSurface(0, 128, 128, 32, 0, 0, 0, 0).contents
         sdl2ext.fill(surface, 0x0)
-        renderer = sdl2ext.RenderContext(surface)
+        renderer = sdl2ext.Renderer(surface)
+        renderer.draw_rect((40, 50, 32, 32), 0x0000FF)
+        view = sdl2ext.PixelView(surface)
+        self.check_lines(view, 128, 128,
+            [((40, 50), (71, 50)),
+             ((40, 50), (40, 81)),
+             ((40, 81), (71, 81)),
+             ((71, 50), (71, 81))], 0x0000FF, (0x0,))
+        del view
+        sdl2ext.fill(surface, 0x0)
+        renderer.draw_rect([(5, 5, 10, 10), (20, 15, 8, 10)], 0x0000FF)
+        view = sdl2ext.PixelView(surface)
+        self.check_lines(view, 128, 128,
+            [((5, 5), (14, 5)),
+             ((5, 5), (5, 14)),
+             ((5, 14), (14, 14)),
+             ((14, 5), (14, 14)),
+             ((20, 15), (27, 15)),
+             ((20, 15), (20, 24)),
+             ((20, 24), (27, 24)),
+             ((27, 15), (27, 24))], 0x0000FF, (0x0,))
+        del view
+
+    @unittest.skipIf(_ISPYPY, "PyPy's ctypes can't do byref(value, offset)")
+    def test_Renderer_fill(self):
+        surface = SDL_CreateRGBSurface(0, 128, 128, 32, 0, 0, 0, 0).contents
+        sdl2ext.fill(surface, 0x0)
+        renderer = sdl2ext.Renderer(surface)
         factory = sdl2ext.SpriteFactory(sdl2ext.TEXTURE, renderer=renderer)
         w, h = 32, 32
         sp = factory.from_color(0xFF0000, (w, h))
@@ -537,6 +694,12 @@ class SDL2ExtSpriteTest(unittest.TestCase):
         view = sdl2ext.PixelView(surface)
         self.check_pixels(view, 128, 128, sp, 0x0000FF, (0x0,))
         del view
+        sdl2ext.fill(surface, 0x0)
+        renderer.fill([(5, 5, 10, 10), (20, 15, 8, 10)], 0x0000FF)
+        view = sdl2ext.PixelView(surface)
+        self.check_areas(view, 128, 128, [(5, 5, 10, 10), (20, 15, 8, 10)],
+                         0x0000FF, (0x0,))
+        del view
 
 
 if __name__ == '__main__':
